from django.core.mail import send_mail as email_sender


def send_mail(subject, message, from_email, recipient_list, **kwargs):
    return email_sender(subject, message, from_email, recipient_list, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from imbox import Imbox

from django_emailsupport import settings


def _create_connection():
    """
    Create new IMAP connection

    :rtype: imbox.Imbox
    """
    connection = Imbox(settings.EMAIL_HOST,
                       username=settings.EMAIL_HOST_USER,
                       password=settings.EMAIL_HOST_PASSWORD,
                       ssl=settings.EMAIL_USE_SSL)

    return connection


def get_unread_messages():
    connection = _create_connection()

    return connection.messages(unread=True)


def mark_seen(uid_list):
    connection = _create_connection()

    for uid in uid_list:
        connection.mark_seen(uid)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import logging

from django.core.management.base import BaseCommand
from django_emailsupport.processor import download_and_save

logger = logging.getLogger('default')


class Command(BaseCommand):
    help = 'Download emails'

    def handle(self, *args, **options):
        try:
            download_and_save()
        except Exception, e:
            logger.error(str(e), extra={'exception': e, 'stack': True})



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Email',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_created=True)),
                ('subject', models.CharField(max_length=256)),
                ('body', models.TextField()),
                ('body_html', models.TextField(blank=True)),
                ('state', models.IntegerField(default=1, choices=[(2, 'Assign'), (1, 'New'), (0, 'Closed')])),
            ],
            options={
                'ordering': ('-created', '-state'),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Submitter',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('address', models.EmailField(unique=True, max_length=75, db_index=True)),
                ('name', models.CharField(max_length=256, null=True, blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='email',
            name='submitter',
            field=models.ForeignKey(to='django_emailsupport.Submitter'),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

from django.db.models import manager


class EmailManager(manager.Manager):

    def get_previous_email(self, email):
        """

        :param email:
        :type email: emailsupport.models.Email
        """

        lookup_params = {'submitter_id': email.submitter_id, 'pk__lt': email.pk}
        try:
            result = self.get_queryset().filter(**lookup_params).order_by('-pk')[:1][0]
        except IndexError:
            result = None

        return result

    def get_next_email(self, email):
        """

        :param email:
        :type email: emailsupport.models.Email
        """

        lookup_params = {'submitter_id': email.submitter_id, 'pk__gt': email.pk}
        try:
            result = self.get_queryset().filter(**lookup_params).order_by('pk')[:1][0]
        except IndexError:
            result = None

        return result



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings

from django_emailsupport.utils.imap import get_unread_messages, mark_seen
from django_emailsupport.settings import NOTIFICATION_EMAIL_LIST
from django_emailsupport.tasks import send_mail

from models import Email, Submitter


def sanitize_email(email):
    return email


def notify_users(email):
    content = email.get_body()
    send_mail(email.subject, content, settings.DEFAULT_FROM_EMAIL, NOTIFICATION_EMAIL_LIST)


def get_submitter_by_email(email, name):

    return Submitter.objects.get_or_create(address=sanitize_email(email),
                                           defaults={'name': name})


def download_and_save():
    email_list = []
    uid_list = []
    for uid, message in get_unread_messages():
        submitter_data = message.sent_from[-1]

        submitter, created = get_submitter_by_email(submitter_data['email'],
                                                    submitter_data['name'])

        email = Email()
        email.submitter = submitter
        email.subject = message.subject
        try:
            email.body = message.body['plain'][-1]
        except IndexError:
            pass
        try:
            email.body_html = message.body['html'][-1]
        except IndexError:
            pass

        email_list.append(email)
        uid_list.append(uid)

    Email.objects.bulk_create(email_list)
    mark_seen(uid_list)

    for email in email_list:
        notify_users(email)



# -*- coding: utf-8 -*-
import os
import sys
import shlex
import subprocess
import django_emailsupport

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages  # NOQA

version = django_emailsupport.__versionstr__


# release a version, publish to GitHub and PyPI
if sys.argv[-1] == 'publish':
    command = lambda cmd: subprocess.check_call(shlex.split(cmd))
    command('git tag v' + version)
    command('git push --tags origin master:master')
    command('python setup.py sdist upload')
    sys.exit()

base_path = os.path.dirname(__file__)


def read_file(filename):
    return open(os.path.join(base_path, filename)).read()

setup(
    name='django-emailsupport',
    version=version,
    description='Simple email support over django admin',
    long_description=read_file('README.rst'),
    author='Martin Voldrich',
    author_email='rbas.cz@gmail.com',
    url='https://github.com/rosti-cz/django-emailsupport',
    packages=find_packages(),
    license=read_file('LICENSE'),
    install_requires=[
        'Django>=1.7', 'imbox',
    ],
    zip_safe=False,
    package_data={'django_emailsupport': ['templates/admin/*', 'static/*/css/*']},
    include_package_data=True,
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet',
    )
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_emailsupport', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Resolution',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('content', models.TextField()),
                ('created', models.DateTimeField(auto_now=True)),
                ('email', models.ForeignKey(to='django_emailsupport.Email')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AlterField(
            model_name='email',
            name='created',
            field=models.DateTimeField(auto_now=True),
        ),
    ]



from __future__ import unicode_literals

from django.db import models
from django.conf import settings
from django.utils.html import strip_tags
from django_emailsupport.tasks import send_mail
from django_emailsupport.manager import EmailManager


User = settings.AUTH_USER_MODEL

EMAIL_STATES = (
    (2, 'Assign'),
    (1, 'New'),
    (0, 'Closed'),
)


class Submitter(models.Model):
    address = models.EmailField(db_index=True, unique=True)
    name = models.CharField(max_length=256, null=True, blank=True)

    def __str__(self):
        return self.address


class Email(models.Model):
    submitter = models.ForeignKey(Submitter)
    subject = models.CharField(max_length=256)
    body = models.TextField()
    body_html = models.TextField(blank=True)
    state = models.IntegerField(choices=EMAIL_STATES, default=1)

    created = models.DateTimeField(auto_now=True)

    objects = EmailManager()

    class Meta:
        ordering = ('-created', '-state')

    def __str__(self):
        return '{} <{}>'.format(self.subject, self.submitter)

    def mark_as_closed(self):
        self.state = 0

    def get_body(self):
        return self.body or strip_tags(self.body_html)


class Resolution(models.Model):
    user = models.ForeignKey(User)
    email = models.ForeignKey(Email)
    content = models.TextField()

    created = models.DateTimeField(auto_now=True)

    def create_email_params(self):
        return {'subject': 'Re: {}'.format(self.email.subject),
                'message': self.get_message_content(),
                'from_email': settings.DEFAULT_FROM_EMAIL,
                'recipient_list': [self.email.submitter.address]}

    def get_message_content(self):
        content = self.email.get_body()
        content = '\r\n> '.join(content.splitlines())
        content = '\r\n> {}'.format(content)
        return '{}\r\n\r\n{}'.format(self.content, content)


def resolution_save_handler(sender, instance, created, **kwargs):
    if created:
        parameters = instance.create_email_params()
        send_mail(**parameters)
        instance.email.mark_as_closed()
        instance.email.save()


models.signals.post_save.connect(resolution_save_handler, Resolution, dispatch_uid='send_email')



from __future__ import unicode_literals

from django.conf import settings


EMAIL_HOST = getattr(settings, 'EMAIL_HOST', None)
EMAIL_HOST_PASSWORD = getattr(settings, 'EMAIL_HOST_PASSWORD', None)
EMAIL_HOST_USER = getattr(settings, 'EMAIL_HOST_USER', None)
EMAIL_PORT = getattr(settings, 'EMAIL_PORT', None)
EMAIL_USE_SSL = getattr(settings, 'EMAIL_USE_SSL', False)

NOTIFICATION_EMAIL_LIST = getattr(settings, 'NOTIFICATION_EMAIL_LIST', None)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

from models import Email, Resolution, Submitter


class ResolutionInline(admin.StackedInline):
    model = Resolution
    max_num = 1


class EmailAdmin(admin.ModelAdmin):
    list_display = ('subject', 'submitter',  'created', 'get_state_display')
    inlines = [ResolutionInline]
    ordering = ('-state', '-created')
    change_form_template = 'admin/email_change_form.html'
    readonly_fields = ('submitter', 'subject', 'body', 'body_html')

    fieldsets = (
        ('Question', {
            'fields': ('submitter', 'subject', 'body', 'body_html', 'state')
        }),
    )

    class Media:
        css = {
            "all": ("admin/css/admin.css",)
        }

    def render_change_form(self, *args, **kwargs):
        response = super(EmailAdmin, self).render_change_form(*args, **kwargs)
        email = response.context_data['original']
        if email:
            response.context_data['previous_email'] = self.get_previous_email(email)
            response.context_data['next_email'] = self.get_next_email(email)
        return response

    def get_previous_email(self, email):
        return Email.objects.get_previous_email(email)

    def get_next_email(self, email):
        return Email.objects.get_next_email(email)


admin.site.register(Email, EmailAdmin)
admin.site.register(Submitter)
